var scientist = require('./scientist');
var EventEmitter = require('events').EventEmitter;
var util = require('util');

var Scout = module.exports = function() {
  this.server = null; // set when app.use initializes it
};

Scout.prototype.createDevice = function(args) {
  var machine = scientist.create.apply(null, args);
  machine._pubsub = this.server.pubsub;
  machine._log = this.server._log;
  machine._registry = this.server.registry;
  return machine;
};

// add a new device to the registry
Scout.prototype.discover = function(constructor) {
  var self = this;
  var machine = this.createDevice(arguments);  
  machine = scientist.init(machine);

  // save device in persistant store
  self.server.registry.save(machine, function(err){
    self.server._jsDevices[machine.id] = machine;
    self.server._log.emit('log','scout', 'Device (' + machine.type + ') ' + machine.id + 'was discovered' );    
    self.server.emit('deviceready', machine);
  });
  return machine;
};

Scout.prototype.provision = function(deviceObject, constructor) {

  // if already initiated on runtime do not create a second instnace
  if(this.server._jsDevices[deviceObject.id]) {
    return null;
  }

  var args = Array.prototype.slice.call(arguments, 1);

  // TODO: add new device code
  var machine = this.createDevice(args);
  machine.id = deviceObject.id; // must set id before machine_config runs
  machine = scientist.init(machine);
  machine.name = deviceObject.name; // must set other properties after machine_config runs

  // add to list of initiated
  this.server._jsDevices[machine.id] = machine;

  this.server.registry.save(machine, function(err){

  });

  this.server._log.emit('log','scout', 'Device (' + machine.type + ') ' + machine.id + 'was provisioned from registry.' );
  this.server.emit('deviceready', machine);
  return machine;
};
